﻿using ProjectSeaplane.DrawningObjects;
using ProjectSeaplane.Generics;
using ProjectSeaplane.MovementStrategy;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectSeaplane
{
    public partial class FormPlaneCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly CarsGenericStorage _storage;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormPlaneCollection()
        {
            InitializeComponent();
            _storage = new CarsGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }
        /// <summary>
        /// Заполнение listBoxObjects
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            foreach (var key in _storage.Keys)
            {
                listBoxStorages.Items.Add(key);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
            >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
        }
        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxObjects_SelectedIndexChanged(object sender,
        EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowPlanes();
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?",
                "Удаление", MessageBoxButtons.YesNo,
                MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString()
                ?? string.Empty);
                ReloadObjects();
            }
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddPlane_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            FormSeaplane form = new FormSeaplane();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (obj + form.SelectedPlane)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowPlanes();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemovePlane_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo,
            MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowPlanes();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRefreshCollection_Click(object sender, EventArgs e)
        {
            {
                if (listBoxStorages.SelectedIndex == -1)
                {
                    return;
                }
                var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
                if (obj == null)
                {
                    return;
                }
                pictureBoxCollection.Image = obj.ShowPlanes();
            }
        }
    }
}
